/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_VED4FUSIONEGOMOTION_H
#define DPC_IMPL_TYPE_VED4FUSIONEGOMOTION_H

#include <cfloat>
#include <cmath>
#include "impl_type_boolean.h"
#include "impl_type_float.h"

namespace dpc {
struct VED4FusionEgoMotion {
    ::Boolean SigStatus{ false };
    ::Float LongAcc{ 0.0F };
    ::Float TimeStamp{ 0.0F };
    ::Float LongVel{ 0.0F };
    ::Float LatYawRate{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(SigStatus);
        fun(LongAcc);
        fun(TimeStamp);
        fun(LongVel);
        fun(LatYawRate);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(SigStatus);
        fun(LongAcc);
        fun(TimeStamp);
        fun(LongVel);
        fun(LatYawRate);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("SigStatus", SigStatus);
        fun("LongAcc", LongAcc);
        fun("TimeStamp", TimeStamp);
        fun("LongVel", LongVel);
        fun("LatYawRate", LatYawRate);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("SigStatus", SigStatus);
        fun("LongAcc", LongAcc);
        fun("TimeStamp", TimeStamp);
        fun("LongVel", LongVel);
        fun("LatYawRate", LatYawRate);
    }

    friend bool operator==(const ::dpc::VED4FusionEgoMotion& lhs, const ::dpc::VED4FusionEgoMotion& rhs) noexcept
    {
        return (lhs.SigStatus == rhs.SigStatus) && (fabs(static_cast<double>(lhs.LongAcc - rhs.LongAcc)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.TimeStamp - rhs.TimeStamp)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.LongVel - rhs.LongVel)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.LatYawRate - rhs.LatYawRate)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_VED4FUSIONEGOMOTION_H
